;******************************************************************************
; File Name   :	SPRACHE2.bin
; Base Address:	0000h Range: 9800h - A700h Loaded length: 0F00h
; reassembliert und kommentiert aus vorliegenden Quelltextfragmenten: 
; Volker Pohlers, September 2007
; orig. Labels sind grogeschrieben
;******************************************************************************

		cpu	z80

;AS-Funktionen
hi              function x,(x>>8)&255
lo              function x, x&255
nextpage	function x, ((x+0ffh)/100h)*100h

;******************************************************************************
;     S P R A C H E R K E N N E R
;
;	fr U880-Rechner
;
;		mit
;	Z9001-Analyse-Modul
;
;  R O B O T R O N  Elektronik Dresden
;
;	Dr.-Ing. Lothar Seveke
;	Dr.-Ing. Gerald Weigert
;
;	Tel.: Dresden 487 2509
;
;******************************************************************************
;
; Kennzeichen der Variante:
;
; - Zeitnormierung
; - direkter Mustervergleich (Hamming-Distanz)
; - maximal 50 Wortklassen
; - fr 2K-EPROM (auer Darst.) und 4K RAM
; - Abtastfrequenz=9,76kHz
; - MV - Komponenten mit 1 Bit gespeichert
; - Lernen mit Clustern
; - Mglichkeit der Darst. der Analyseergebnisse
;
;------------------------------------------------------------------------------

;VER		equ	'SPRACHE1'
VER		equ	'SPRACHE2'


		IF VER='SPRACHE1'
ANF		equ	6000h		; SPRACHE1
		ELSEIF VER='SPRACHE2'
ANF		equ	9800h		; SPRACHE2
		ENDIF

		org 	ANF

; Speicher
P1ROL		equ	003Bh
AREC		equ	004Ch
AREC2		equ	004Eh
STAT		equ	0050h
ERKER		equ	0051h		; bergabe fr Erkennungergebnis
FCB		equ	005Ch
CONBU		equ	0080h
;
CTK0		equ	0210h		; Beginn Interrupttabelle fr 3 CTC-Kanle

; I/O
CTC0		equ	0E0h
CTC1		equ	0E1h
CTC3		equ	0E3h
LEDON		equ	0E4h
LEDOFF		equ	0E8h

;------------------------------------------------------------------------------
; Programmkonstanten
;------------------------------------------------------------------------------

; Rckweisungsschwelle
; (grer --> mehr Rckweisungen)
; (  = 0  --> keine Rckweisungen)
RUESW		equ	30

; Schwellwert fr das Streichen von Clustern
; kleiner --> mehr Muster/Klasse
; (=8 --> 32 Cluster / Klasse,
; =64 -->  4 Cluster / Klasse)
SWCLU		equ	8

SWRUE		equ	RUESW
SW1		equ	40
T1		equ	2
SW2		equ	40
T2		equ	8
SW3		equ	40
T3		equ	T1
;VDIM		equ	8
;ZNOR		equ	16
;ZBYTV		equ	1
SWHL		equ	32
KLZAHL		equ	50
N0		equ	128
CSCHW		equ	15		; Clusterschwelle
NANF		equ	64		; Anfangsbewertung bei Erffnung eines neuen Clusters
NDELT		equ	32
KGREN		equ	4
DIM		equ	8

;------------------------------------------------------------------------------
; Start
;------------------------------------------------------------------------------

		jp	START
		IF VER='SPRACHE1'
		db	"SPRACHE1",0
		ELSEIF VER='SPRACHE2'
		db	"SPRACHE2",0
		ENDIF
		db    0

START:		ld	hl, RECOG
		ld	(AREC), hl
		ld	hl, RECO2
		ld	(AREC2), hl
		ld	a, 0FFh
		ld	(STAT), a
		ld	a, (loadflg)
		and	a
		call	z, mDel2

start1:		ld	a, 0
		ld	(P1ROL), a
;
		ld	de, aMusteranzahlIm ; "\fMusteranzahl im Wortschatz: "
		ld	a, (KLAZ)	; Anzahl der Cluster
		call	ANZM		; Anzeige Text Reg. DE und Anzeige Reg.	A dezimal
		ld	de, aEingestellterT ; "\r\n\nEingestellter Teilwortschatz: "
		ld	a, (twz)
		call	ANZM		; Anzeige Text Reg. DE und Anzeige Reg.	A dezimal
		ld	de, aMenue	; "\r\n\n\n  A   Wortschatz laden\r\n\n  B   Lern"...
		call	STROUT		; Textausgabe
start2:		ld	de, aKennbuchstabe ; "Kennbuchstabe:  \b\b"
		call	STROUT		; Textausgabe
		ld	a, 2
		call	ETAST		; Texteingabe nach DE
		jr	c, start4
		cp	1
		jr	nz, start3	; Fehler: nicht	nur 1 Zeichen
		ld	a, (de)
		cp	'A'
		jr	c, start3	; Fehler: kein gltiges	Zeichen
		cp	'H'
		jr	nc, start3	; Fehler: kein gltiges	Zeichen
		call	CLS
		ld	hl, start1
		push	hl
		cp	'A'
		jp	z, mLoad
		cp	'C'
		jp	z, mSave
		cp	'D'
		jp	z, mModify
		cp	'G'
		jp	z, mDel
		push	af
		ld	de, aTeilwortschatz ; "Teilwortschatz: "
		ld	a, (twz)
		call	ANZM		; Anzeige Text Reg. DE und Anzeige Reg.	A dezimal
		call	OUT2CRLF	; 2x Enter ausgeben
		ld	a, 2
		ld	(P1ROL), a
		pop	af
		cp	'B'
		jp	z, mLearn
		cp	'F'
		jp	z, mDelClass
		jp	mRename
;
start3:		call	CUUP		; Cursor hoch
		jr	start2
;
start4:		call	OUT2CRLF	; 2x Enter ausgeben
		and	a
		ret
;
aMusteranzahlIm:db	0Ch,"Musteranzahl im Wortschatz: ",0
aEingestellterT:db	"\r\n"
		db	"\n"
		db	"Eingestellter Teilwortschatz: ",0
aMenue:		db	"\r\n"
		db	"\n"
		db	"\n"
		db	"  A   Wortschatz laden\r\n"
		db	"\n"
		db	"  B   Lernen\r\n"
		db	"\n"
		db	"  C   Wortschatz abspeichern\r\n"
		db	"\n"
		db	"  D   Teilwortschatz aendern\r\n"
		db	"\n"
		db	"  E   Klassen umbenennen\r\n"
		db	"\n"
		db	"  F   Klassen loeschen\r\n"
		db	"\n"
		db	"  G   Wortschatz loeschen\r\n"
		db	"\n"
		db	"STOP  Programmende\r\n"
		db	"\n"
		db	"\n",0
aKennbuchstabe:	db	"Kennbuchstabe:  \b\b",0
aTeilwortschatz:db	"Teilwortschatz: ",0

;------------------------------------------------------------------------------
; Wortschatz laden
;------------------------------------------------------------------------------

mLoad:		ld	a, (KLAZ)	; Anzahl der Cluster
		and	a
		jr	z, mLoad1
		ld	de, aWortschatzNich ; "Wortschatz nicht geloescht\r\n"
		call	STROUT		; Textausgabe
		jp	OUT_ENTER	; Ausgabe von ">ENTER<" und Eingabe ein Zeichen

mLoad1:		call	filename	; Dateiname erfragen und in FCB	eintragen
		ret	c
		ld	de, CONBU
		ld	c, 26		; SETDM	Setzen der Pufferadresse fr Kassette lesen/schreiben
		call	5

mLoad2:		ld	c, 13		; OPENR	Erffnen Kassette lesen
		call	5
		jr	nc, mLoad3
		and	a
		ret	z
		call	0F5A6h		; OS: REA1, WARTEN AUF BEDIENERREAKTION
		ret	c
		jr	mLoad2

mLoad3:		xor	a
		ld	(loadflg), a
		ld	de, KLAZ	; Anzahl der Cluster
		ld	c, 26		; SETDM	Setzen der Pufferadresse fr Kassette lesen/schreiben
		call	5
		ld	b, 29		; Anzahl Blcke	(29*128	Byte)

mLoad4:		ld	c, 20		; READS	Lesen eines Blockes von	Kassette
		call	5
		jr	c, mLoad6
		or	a
		jr	nz, mLoad5
		djnz	mLoad4
		jr	mLoad7

mLoad5:		ld	a, 1
		ld	(loadflg), a
		ret

mLoad6:		call	0F5A6h		; OS: REA1, WARTEN AUF BEDIENERREAKTION
		jr	nc, mLoad4
		jp	mDel2

mLoad7:		call	mDel2

mLoad8:		ld	de, aFilelaengenFeh ; "\r\nFilelaengen-Fehler\r\n"
		call	STROUT		; Textausgabe
		jp	OUT_ENTER	; Ausgabe von ">ENTER<" und Eingabe ein Zeichen

aWortschatzNich:db	"Wortschatz nicht geloescht\r\n",0
aFilelaengenFeh:db	"\r\n"
		db	"Filelaengen-Fehler\r\n",0

;------------------------------------------------------------------------------
; Wortschatz loeschen
;------------------------------------------------------------------------------

mDel:		ld	de, aWortschatzLoes ; "Wortschatz loeschen J/N ?  \b\b"
		call	STROUT		; Textausgabe
		ld	a, 2
		call	ETAST		; Texteingabe nach DE
		ret	c
		cp	1
		jr	nz, mDel1
		ld	a, (de)
		cp	'J'
		jr	z, mDel2
		cp	'Y'
		jr	z, mDel2
		cp	'N'
		ret	z
;
mDel1:		call	CUUP		; Cursor hoch
		jr	mDel
;
mDel2:		xor	a
		ld	(KLAZ),	a	; Anzahl der Cluster
		ld	(ANSYM), a
		dec	a
		ld	(ANSYM+1), a
		ld	(loadflg), a
		ret

aWortschatzLoes:db	"Wortschatz loeschen J/N ?  \b\b",0


;------------------------------------------------------------------------------
; Dateiname erfragen und in FCB	eintragen
;------------------------------------------------------------------------------

filename:
;Dateinamen lschen
		xor	a
		ld	b, 8
		ld	hl, 5Ch	; '\'   ; FCB

filen1:		ld	(hl), a
		inc	hl
		djnz	filen1
;Dateiendung "WOR"
		ld	(hl), 'W'
		inc	hl
		ld	(hl), 'O'
		inc	hl
		ld	(hl), 'R'
;
filen2:		ld	de, aFilename	; "Filename:         \b\b\b\b\b\b\b\b\b"
		call	STROUT		; Textausgabe
		ld	a, 9
		call	ETAST		; Texteingabe nach DE
		ret	c
		jr	z, filen_err	; Fehler: kein Zeichen eingegeben
		cp	9
		jr	nc, filen_err	; Fehler: zu viel Zeichen eingegeben
		ld	b, a
		ld	hl, FCB
		ld	a, (de)
		cp	41h ; 'A'
		jr	c, filen_err	; Fehler: ungltiges Zeichen
		cp	7Fh ; ''
		jr	nc, filen_err	; Fehler: ungltiges Zeichen
		ld	(hl), a
		dec	b
		jr	z, filen4
;
filen3:		inc	hl
		inc	de
		ld	a, (de)
		ld	(hl), a
		djnz	filen3
;
filen4:		and	a
		ret
;
filen_err:	call	CUUP		; Cursor hoch
		jr	filen2

aFilename:	db	"Filename:         \b\b\b\b\b\b\b\b\b",0

;------------------------------------------------------------------------------
; Teilwortschatz aendern
;------------------------------------------------------------------------------

mModify:	ld	de, aTeilwortscha_0 ; "Teilwortschatz:    \b\b\b\b"
		ld	a, (twz)
		call	ANZM		; Anzeige Text Reg. DE und Anzeige Reg.	A dezimal
		ld	a, 2
		call	ETAST		; Texteingabe nach DE
		ret	c
		ret	z
		call	DEZHX		; Umwandlung BCD-Zahl (A Stellen Text ab DE) nach HL
		jr	c, mModify2
		and	a
		jr	z, mModify2
		cp	6
		jr	c, mModify3

mModify2:	call	CUUP		; Cursor hoch
		jr	mModify

mModify3:	ld	(twz), a
		ld	b, a
		xor	a

mModify4:	dec	b
		jr	z, mModify5
		add	a, 50
		jr	mModify4

mModify5:	ld	(SPRIN), a	; Sprecherindex
		ret

aTeilwortscha_0:db	"Teilwortschatz:    \b\b\b\b",0

;------------------------------------------------------------------------------
; Lernen
;------------------------------------------------------------------------------

mLearn:		ld	de, aSprechen	; "Sprechen:  "
		call	STROUT		; Textausgabe
		ld	a, (SPRIN)	; Sprecherindex
		ld	(SPRI1), a
		call	REC1
		bit	7, a
		ret	nz

mLearn1:	call	LPMOD
		ld	de, aErkannteKlasse ; "Erkannte Klasse:         \b\b\b\b\b\b\b\b"
		call	ANZM		; Anzeige Text Reg. DE und Anzeige Reg.	A dezimal
		ld	a, 3
		call	ETAST		; Texteingabe nach DE
		ret	c
		jr	z, mLearn4
		call	DEZHX		; Umwandlung BCD-Zahl (A Stellen Text ab DE) nach HL
		jr	nc, mLearn2
		call	CUUP		; Cursor hoch
		ld	de, aSprechen	; "Sprechen:  "
		call	STROUT		; Textausgabe
		jr	mLearn1

mLearn2:	and	a
		jr	z, mLearn3
		ld	b, a
		ld	a, (SPRIN)	; Sprecherindex
		add	a, b
mLearn3:	ld	(XSYM),	a
mLearn4:	ld	a, (XSYM)
		and	a
		jr	z, mLearn
		call	CLUST
		ld	a, (KLAZ)	; Anzahl der Cluster
		cp	201
		jr	c, mLearn
		ld	de, aWortschatzVoll ; "\nWortschatz voll\r\n"
		call	STROUT		; Textausgabe
		dec	a
		ld	(KLAZ),	a	; Anzahl der Cluster
		ld	a, 0FFh
		ld	(unk_A757), a
		jp	OUT_ENTER	; Ausgabe von ">ENTER<" und Eingabe ein Zeichen

aSprechen:	db	"Sprechen:  ",0
aErkannteKlasse:db	"Erkannte Klasse:         \b\b\b\b\b\b\b\b",0
aWortschatzVoll:db	"\n"
		db	"Wortschatz voll\r\n",0

;------------------------------------------------------------------------------
; Erk.-Progr. A : Startadresse in Zelle 76 (bzw. 4CH)
; Nach Aufruf dieses Erkennerprogramms wird auf ein
; gltiges Wort gewartet, d.h., bei Rckweisung
; (Klassennummer 0) wird ein BEEP-Signal ausgegeben und
; weiter auf ein gltiges Wort gewartet.
;------------------------------------------------------------------------------

RECOG:		push	bc
		push	de
		push	hl
		push	ix
		push	iy
		ex	af, af'
		push	af
		ex	af, af'
		exx
		push	bc
		push	de
		push	hl
		ld	a, (STAT)
		cp	0FFh
		jr	z, recog2
		ld	b, a
		xor	a

recog1:		dec	b
		jr	z, recog2
		add	a, 32h ; '2'
		jr	recog1

recog2:		ld	(SPRI1), a

recog3:		call	REC1
		ld	(ERKER), a
		or	a
		jr	nz, recoge
		call	BEEP
		jr	recog3

recoge:		pop	hl
		pop	de
		pop	bc
		exx
		ex	af, af'
		pop	af
		ex	af, af'
		pop	iy
		pop	ix
		pop	hl
		pop	de
		pop	bc
		ret

;------------------------------------------------------------------------------
;Erk.-Progr. B : Startadresse in Zelle 78 (bzw. 4EH)
;Kommt nach jedem erkannten Wort mit der
;Klassennummer (auch mit Klassennummer 0) zurck.
;------------------------------------------------------------------------------

RECO2:		push	bc
		push	de
		push	hl
		push	ix
		push	iy
		ex	af, af'
		push	af
		ex	af, af'
		exx
		push	bc
		push	de
		push	hl
		ld	a, (STAT)
		cp	0FFh
		jr	z, reco22
		ld	b, a
		xor	a

reco21:		dec	b
		jr	z, reco22
		add	a, 50
		jr	reco21

reco22:		ld	(SPRI1), a
		call	REC1
		ld	(ERKER), a
		jr	recoge



;------------------------------------------------------------------------------
; Unterprogramm zur Erkennung eines Wortes
;
; Eingabebereitschaft mit Aufruf des UP
; Ausgang: A-Register = Symbol des erkannten Wortes
; 	 Bit 7 = 1 --> Tastatureingabe erfolgt
; XSYM = Symbol des erkannten Wortes
;------------------------------------------------------------------------------

REC1:		ld	(RETT),	sp
		ld	sp, SPERK
		call	ANAL

REC2:		call	INKEY		; Tastaturabfrage
		jr	z, REC4
		ld	a, (AZUST)
		and	7
		jr	z, REC2		; Warten auf Wortende oder berlauf XFELD
		jr	INTL1

INTL:
ADWEP:		ld	sp, SPERK	; SPERK	Einsprung mit prognostiziertem Wortende

INTL1:		call	ZEIN
		call	VGLEI
		ld	a, (IDEX1)	; Laufende Nummer des Symbols der
		ld	e, a		; erkannten Klasse in der Tabelle der Anzeigesymbole
		ld	d, 0
		ld	hl, ANSYM	; Tabelle der Anzeigesymbole
		add	hl, de
		ld	a, (hl)		; Anzeige Symbol der erk. Klasse
		ld	(XSYM),	a

REC3:		call	INKEY		; Tastaturabfrage
		jr	z, REC4
		ld	a, (AZUST)
		and	3
		jr	z, REC3		; Warten auf sicheres Wortende oder berlauf
		call	LPMOD

REC4:		ld	sp, (RETT)
		call	INTLO
		ret


;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

INKEY:		ld	a, 0BFh
		out	(90h), a
		nop
		in	a, (91h)
		cp	0BFh
		ret	nz
		di
		out	(LEDOFF), a
		set	7, a
		ret

;------------------------------------------------------------------------------
;Ermittlung der	wirklichen Klassennummer (1..50)
;durch Beseitigung der Probenkennung(0,50,...N*KMAX)
;------------------------------------------------------------------------------

LPMOD:		push	bc
		ld	b, KLZAHL
		ld	a, (XSYM)

LPM2:		sub	b
		jr	z, LPM1
		jr	nc, LPM2

LPM1:		add	a, b
		pop	bc
		ret




;******************************************************************************
;         Spracherkenner SE 9001
;       Kommentierte Unterprogramme
;
;            (c) PHONEI 1986
;******************************************************************************


;------------------------------------------------------------------------------
; Clusterbildung
;
; Das in YFELD abgespeicherte Signalmuster fhrt zu einem neuen Cluster,
; wenn XSYM noch nicht in ANSYM vorhanden ist oder die minimale H.-Distanz
; des Signalmusters zu allen in RFELD abgespeicherten Mustern mit der
; Bezeichnung XSYM grer als CSCHW ist. Andernfalls wird die zum hnlichsten
; gehrende Clusterbewertung um NDELT erhht.
;
;	MINHD:	Minimale H.-Distanz
;	CSCHW:	Clusterschwelle
;	INDEX:	relative Verschiebung in ANSYM
;	KLAZ:	Anzahl der Cluster
;	XSYM:	Klassensymbol
;	NANF:	Anfangsbewertung bei Erffnung eines neuen Clusters
;	RFELD:	Feld der Referenzmuster
;	YFELD:	Normiertes Signal
;	NDELT:	Increment der Clusterbewertung
;	BFELD:	Feld der Clusterbewertungen
;	SPRIN:	Sprecherindex (0,KMAX,...,KMAX(SPMAX-1()
;	KMAX:	maximale Klassenzahl
;
;------------------------------------------------------------------------------


CLUST:		ld	a, 0FEh
		ld	(SPRI1), a
		call	VGLEI
		ld	a, (MINHD)
		ld	b, a
		ld	a, CSCHW
		cp	b
		jr	nc, CL1		; kein neues Cluster
; neues Cluster
;
; Clusterzahl um 1 erhhen
		ld	hl, KLAZ	; Anzahl der Cluster
		inc	(hl)
; Symbol XSYM an Feld ANSYM anfgen
		ld	d, 0
		ld	e, (hl)
		ld	hl, ANSYM
		add	hl, de
		call	LPMOD
		ld	b, a
		ld	a, (SPRIN)	; Sprecher- oder Sprachprobenindex
		add	a, b
		ld	(hl), a
		ld	a, 0FFh
		inc	hl
		ld	(hl), a		; neues	Ende-Kennzeichen
; Anfangsclusterbewertung an BFELD anfgen
		ld	hl, BFELD
		dec	de		; DE: KLAZ-1
		add	hl, de		; HL: Zeiger in	BFELD
		ld	a, NANF
		ld	(hl), a
; Muster an RFELD anfgen
		ex	de, hl
		add	hl, hl
		add	hl, hl
		add	hl, hl
		add	hl, hl
		ex	de, hl		; DE: 16*(KLAZ-1)
		ld	hl, RFELD
		add	hl, de
		ex	de, hl		; DE: Ziel in RFELD
		ld	hl, YFELD	; HL: Quelle in	YFELD
		ld	bc, 16		; BC: Byte-Zhler in YFELD
		ldir
		jr	CL2
;
CL1:		ld	hl, XSYM
		ld	a, (SPRIN)	; Sprecherindex
		cp	(hl)
		jr	nc, CL3		; XSYM < SPRIN
		ld	d, a
		ld	a, KLZAHL
		add	a, d
		cp	(hl)
		jr	c, CL3		; XSYM > SPRIN+KMAX
		ld	hl, BFELD-1
		ld	a, (IDEX1)	; INDEX
		ld	d, 0
		ld	e, a
		add	hl, de		; HL: Zeiger in	BFELD
		ld	a, NDELT
		add	a, (hl)
		ld	(hl), a
; Klasse normieren
CL2:		call	KNORM
		call	CLST
		call	KNORM
;
CL3:		ret


;------------------------------------------------------------------------------
; Cluster streichen
;
; Alle Cluster mit der Klassenbezeichnung XSYM werden in
; ANSYM, BFELD und RFELD gestrichen, wenn ihre Bewertung
; den Bewertungsschwellwert NSCHW unterschreitet
;
;	XSYM:	Klassensymbol
;	ANSYM:	Feld der Anzeigesymbole
;	BFELD:	Feld der Clusterbewertungen
;	RFELD:	Feld der Referenzmuster
;	NSCHW:	Schwellwert der	Clusterbewertungen
;	KLAZ:	Anzahl der Cluster
;
;------------------------------------------------------------------------------

CLST:		ld	a, (KLAZ)	; Anzahl der Cluster
		inc	a
		ld	b, 0
		ld	c, a		; BC: Cluster-Zhler
		ld	de, 0FFFFh	; DE: rel. Verschiebung	zur Anfangsadresse
		ld	hl, ANSYM+1 	; HL: Zeiger in ANSYM
CR11:		ld	a, (XSYM)
; XSYM in ANSYM suchen
CR1:		cpi
		inc	de
		ret	po		; alle Cluster-Symb. verarb. --	Ausgang	Clstr
		jr	nz, CR1		; nchster Cluster
; Bewertung des gefundenen Clusters mit NSCHW vergleichen
		push	hl
		ld	hl, BFELD	; Schwelle: Cluster streichen
		add	hl, de		; Zeiger in BFELD
		ld	a, (NSCHW)
		cp	(hl)
		pop	hl
		jr	c, CR11		; Cluster ist gltig
; Cluster in ANSYM streichen
		push	hl		; HL: Quelle in	ANSYM
		push	bc		; BC: Byte-Zhler in ANSYM
		push	de
		ld	d, h
		ld	e, l
		dec	de		; DE: Ziel in ANSYM
		ldir
		pop	de
		pop	bc
; Cluster in BFELD steichen
		push	bc
		push	de
		dec	bc		; BC: Byte-Zhler in BFELD
		ld	a, b
		or	c
		jr	z, CR2
		ld	hl, BFELD
		add	hl, de
		ld	d, h
		ld	e, l		; DE: Ziel in BFELD
		inc	hl		; HL: Quelle in	BFELD
		ldir
		pop	de
		pop	bc
; Cluster in RFELD streichen
		push	bc
		push	de
		ex	de, hl
		add	hl, hl
		add	hl, hl
		add	hl, hl
		add	hl, hl
		ex	de, hl		; DE: =16*DE
		dec	bc
		ld	h, b
		ld	l, c
		add	hl, hl
		add	hl, hl
		add	hl, hl
		add	hl, hl
		ld	b, h
		ld	c, l		; BC:=16*BC, Byte-Zhler in RFELD
		ld	hl, RFELD
		add	hl, de
		push	hl
		ld	de, 16
		add	hl, de		; HL: Quelle in	RFELD
		pop	de		; DE: Ziel in RFELD
		ldir
CR2:		ld	hl, KLAZ	; Anzahl der Cluster
		dec	(hl)
		pop	de
		pop	bc
		pop	hl
; Feld ANYM ist um ein Symbol verkrzt
		dec	hl
		dec	de
		jr	CR11		; nchster Cluster



;------------------------------------------------------------------------------
; Clusterbewertung normieren
;
; Normieren der Clusterbewertungen der Klasse XSYM
; Die Summe aller Clusterbewertungen dieser Klasse betrgt N0
;
;	KLAZ:	Anzahl der Cluster
;	XSYM:	Klassensymbol
;	ANSYM:	Feld der Anzeigesymbole
;	BFELD:	Feld der Clusterbewertungen
;	N0:	Norm der Klassenbewertung
;
;------------------------------------------------------------------------------

KNORM:		xor	a
		ld	(NSUM),	a
;
		ld	b, a
		ld	a, (KLAZ)	; Anzahl der Cluster
		inc	a
		ld	c, a		; BC: Cluster-Zhler
		ld	hl, ANSYM+1 	; HL: Zeiger in	ANSYM
		ld	de, BFELD-1 	; DE: Zeiger in BFELD
		push	bc
		push	de
		push	hl
KM11:		ld	a, (XSYM)
; 1. Durchlauf
; Cluster-Bewertungen der Klasse XSYM in NSUM akkumulieren
KM1:		cpi
		inc	de
		jp	po, KM2		; alle Cluster verarbeitet
		jr	nz, KM1		; Symbol ungleich XSYM
;
		ld	a, (NSUM)
		ex	de, hl
		add	a, (hl)
		ex	de, hl
		ld	(NSUM),	a
		jr	KM11		; nchstes Cluster
; 2. Durchlauf
; Clusterbewertungen der Klasse XSYM mit N0/NSUM multiplizieren

KM2:		pop	hl
		pop	de
		pop	bc
KM55:		ld	a, (XSYM)
KM5:		cpi
		inc	de
		ret	po		; alle Cluster verarb.--Ausgang	KNORM
		jr	nz, KM5		; Symbol ungleich XSYM
;
		push	hl
		push	de
		ex	de, hl
		ld	a, N0
		ld	e, a
		ld	a, (hl)
		call	MUL1		; Clusterbewertung mit N0 multiplizieren
		ld	a, (NSUM)
		ld	d, a
		call	DIV1		; Clusterbewertung durch NSUM dividieren
		pop	hl
		pop	de
		ld	(hl), a
		ex	de, hl
		jr	KM55		; nchstes Cluster


;------------------------------------------------------------------------------
; Multiplikation vorzeichenloser Zahlen
; HL=A*E
; Bedingung: Faktoren kleiner 256
;------------------------------------------------------------------------------

MUL1:		push	bc
		push	de
		ld	d, 0
		ld	b, 8
		ld	hl, 0
mul11:		add	hl, hl
		rla
		jr	nc, mul12
		add	hl, de
mul12:		djnz	mul11
		pop	de
		pop	bc
		ret

;------------------------------------------------------------------------------
; Division vorzeichenloser Zahlen
; A=HL/D
; Bedingung: Ergebnis und Divisor kleiner 256
;------------------------------------------------------------------------------

DIV1:		push	bc
		push	de
		ld	e, 0
		ld	b, 8
DIV11:		srl	d
		rr	e
		sbc	hl, de
		rla
		jp	p, DIV12
		add	hl, de
DIV12:		djnz	DIV11
		cpl
		pop	de
		pop	bc
		ret


;------------------------------------------------------------------------------
; Vergleichen
;
; Berechnung der minimalen Hamming-Distanz zwischen
; einem normierten Signal und den Referenzmustern
;
; 	YFELD:	Feld des normierten Signals
; 	RFELD:	Feld der Referenzmuster
; 	HTAB:	Tabelle	der H.-Distanzen (1/4K-Grenze)
; 	KLAZ:	Anzahl der Cluster
; 	MINHD:	minimale Hamming-Distanz
; 	INDEX:	(ANSYM+INDEX)-->gesuchtes Klassensymbol
; 	XSYM:	Klassensymbol, fr dessen cluster MINHD und
; 		Indes berechnet	werden,
; 		XSYM=0:	Es werden alle Cluster einbezogen
; 			(normaler Erkennerbetrieb)
;
;------------------------------------------------------------------------------

VGLEI:		ld	hl, HTAB	; HL: Zeiger in	HTAB
		ld	ix, RFELD	; IX: Zeiger in	RFELD
		ld	a, (KLAZ)	; Anzahl der Cluster
		ld	c, a		; C: Cluster-Zhler
		ld	e, 0FFh		; E: minimale H.-Distanz
		xor	a
		cp	c
		jr	z, vgl6		; Referenzspeicher leer
;
VGL2:		ld	iy, YFELD	; IY: Zeiger in	YFELD
		ld	b, 16		; B: Vektorzhler
		ld	d, 0		; D: aktuelle H.-Distanz
;Ermittlung der	aktuellen H.-Distanz/Klasse
VGL1:		ld	a, (iy+0)
		xor	(ix+0)
;
		scf			; oder NOP bei groer HTAB
		rra			; oder NOP ...
;
		ld	l, a
		ld	a, (hl)
		adc	a, d
		ld	d, a
		inc	iy
		inc	ix
		djnz	VGL1		; neues	Byte
;
		ld	a, (SPRI1)
		cp	0FFh
		jr	z, VGL4		; alle Cluster bercksichtigen
; nur Cluster der Klasse XSYM bercksichtigen
		push	hl
		push	de
		ld	hl, ANSYM
		ld	d, 0
		ld	a, (KLAZ)	; Anzahl der Cluster
		ld	e, a
		add	hl, de
		ld	e, c
		xor	a
		sbc	hl, de
		pop	de
		inc	hl		; HL: Zeiger in	ANSYM
;
		ld	a, (SPRI1)
		cp	0FEh
		jr	nz, vgl40
		ld	a, (XSYM)
		cp	(hl)
		pop	hl
		jr	z, VGL4
		jr	VGL3
;
vgl40:		ld	b, (hl)
		pop	hl
		cp	b
		jr	nc, VGL3
		push	de
		ld	d, a
		ld	a, 50
		add	a, d
		pop	de
		cp	b
		jr	c, VGL3
; Vergleich aktuelle H.-Distanz mit minimaler H.-Distanz
VGL4:		ld	a, d
		cp	e
		jr	nc, VGL3	; D nicht keiner E
		ld	e, a
		ld	a, c
		ld	(INDEX), a	; INDEX:=C
VGL3:		dec	c
		jr	nz, VGL2
; Rckweisungstest
		ld	a, SWRUE
		cp	e
		jr	nc, VGL5	; Keine	Rckweisung
		xor	a
		jr	vgl6
; richtigen Klassenindex ermitteln
VGL5:		ld	a, (INDEX)
		ld	c, a
		ld	a, (KLAZ)	; Anzahl der Cluster
		sub	c
		inc	a
vgl6:		ld	(IDEX1), a
;minimale H.-Distanz abspeichern
		ld	a, e
		ld	(MINHD), a
		ret


; ------------------------------------------------------------------------------
; Zeitliche Normierung
;
; ein Signal, bestehend aus VANZ Vektoren mit je 8 Bit-Komponenten
; wird auf eine Lnge von 16 Vektoren normiert
;
; 	VANZ:	Vektoranzahl im	Signal
; 	XFELD:	Eingangssignal
; 	YFELD:	Ausgangssignal
; 	ZZ:	zhlt fr die Komponenten 0...7	der YFELD-Vektoren die gesetzten Bits
; 	XDIFF:	Zahl der Verarbeitungsschritte fr XFELD-Vektor
; 	YDIFF:	Zahl der Verarbeitungsschritte fr YFELD-Vektor
;
;------------------------------------------------------------------------------

ZEIN:		ld	ix, XFELD	; IX: Zeiger in	XFELD
		ld	iy, YFELD	; IY: Zeiger in	YFELD
;
		ld	a, 16
		ld	(XDIFF), a
		ld	b, a		; B: Vektorzhler in YFELD
;
ZN00:		ld	a, (VANZ)
		ld	(YDIFF), a
		sra	a
		ld	(ZZ), a
		ld	hl, ZZ		; HL: Quelle in	ZZ
		ld	d, h
		ld	e, l
		inc	de		; DE: Ziel in ZZ
		push	bc
		ld	bc, 7		; BC: Byte-Zhler in ZZ
		ldir
		pop	bc
;
ZN0:		ld	hl, ZZ
		ld	a, (YDIFF)
		ld	c, a
		ld	a, (XDIFF)
		sub	c
		jr	nc, ZN200
; XFELD-Vektor wird vollstndig verarbeitet
		ld	a, (ix+0)
		ld	c, a		; C: XFELD-Vektor
		ld	a, (XDIFF)
		ld	d, a		; D: Decrement
		push	bc
		ld	b, 8		; B: Komponentenzhler
;
ZN2:		rl	c
		jr	nc, ZN1		; Bit nicht gesetzt
		ld	a, (hl)
		sub	d
		ld	(hl), a
ZN1:		inc	hl
		djnz	ZN2		; nchste Komponente
;
		pop	bc
		inc	ix
		ld	a, (YDIFF)
		sub	d
		ld	(YDIFF), a
		ld	a, 16
		ld	(XDIFF), a
		jr	ZN0		; nchster XFELD-Vektor
; YFELD-Vektor wird vollstndig verarbeitet
ZN200:		ld	a, (ix+0)
		ld	c, a		; C: YFELD-Vektor
		ld	a, (YDIFF)
		ld	d, a		; D:Decrement
		push	bc
		ld	b, 8		; B: Komponentenzhler
;
ZN8:		ld	a, (hl)
		rl	c
		jr	nc, ZN9		; Bit nicht gesetzt
		sub	d
ZN9:		rla
		rl	e		; E: YFELD-Vektor aufbauen
		inc	hl
		djnz	ZN8		; nchste Komponente
;
		pop	bc
		ld	(iy+0),	e
		inc	iy
		ld	a, (XDIFF)
		sub	d
		ld	(XDIFF), a
;
		djnz	ZN00		; nchster YFELD-Vektor
;
		ret


;------------------------------------------------------------------------------
; Initialisierung der NDG-Analyse
; Taktfrequenz=2,5MHz => Mefrequenz=09,766kHz
; Zeitfensterlnge=26,2ms
;------------------------------------------------------------------------------

;Steuerworte fr E/A-Initialisierung
;
;Flankenmelder Tiefpa
TAINI:		db CTC0
		db    3
		db lo(CTK0)
		db 0D5h
		db    1
;NDG-Abstand / Zeitfenster
		db CTC1
		db    2
		db 0B5h
		db 0FFh
;Flankenmelder Hochpa
		db CTC3
		db    2
		db 0D5h
		db    1
;Adressen der Interruptroutinen	bei aktiver Analyse
TADRA:		dw TP
		dw ZUL
		dw 0
TADRE:		dw HP

;
;UP zur	Vorbereitung der Analyse
;
ANAL:		di
		ld	a, 3
		out	(82h), a
		out	(83h), a
		out	(92h), a
		out	(93h), a
;init.Verbindung Z.laden
		ld	hl, TADRA
		ld	de, CTK0
		ld	bc, 8		; TADRE-TADRA+2
		ldir
;Ausgangszustand herstellen
		ld	hl, VPUF1
		ld	b, 25		; ?XFELD-VPUF1?
		xor	a

ANA1:		ld	(hl), a
		inc	hl
		djnz	ANA1
;
		ld	(AZUST), a
		ld	a, T1
		ld	(TT1), a
		ld	(TT3), a
		ld	a, T2
		ld	(TT2), a
;
		ld	hl, XFELD
		ld	(XA), hl
;
		exx
		ld	hl, VPUF1
		ld	d, hi(KTAB)
		ld	bc, 0
		exx
;
;E/A Ports initialisieren
;
		ld	hl, TAINI
		ld	e, 3

INI1:		ld	c, (hl)
		inc	hl
		ld	b, (hl)
		inc	hl
		otir
		dec	e
		jr	nz, INI1
		out	(LEDON), a
		ei
		ret
;
;Interrupts paralysieren
;
INTLO:		push	af
		ld	a, 73h
		out	(CTC0),	a
		out	(CTC1),	a
		out	(CTC3),	a
;
		ld	c, 25		; INITA	Initialisierung	der Tastatur und der Systemuhr
		call	5
;
		ld	hl, 1000h
INTLO1:		dec	hl
		ld	a, h
		or	l
		jr	nz, INTLO1
;
		ld	c, 11		; CSTS Abfrage Status CONST
		call	5
		ld	c, 1		; CONSI	Eingabe	eines Zeichens von CONST
		and	a
		call	nz, 5
		pop	af
		ret

;------------------------------------------------------------------------------
; Interrupts Analyse
;
; Bedingungen: max. Zhlwert=0FFh
;  B':     aktueller Zhlerstand HP
;  C':     aktueller Zhlerstand TP
;  HL':    Zeiger im aktuellen Vektorpuffer
;  DE':    Zeiger in KTAB
;  AZUST:  Analyse-Zustand
;------------------------------------------------------------------------------

ZUL:		exx
		ex	af, af'
;
		ld	a, (AZUST)
		rla
		jr	c, ZUL1		; kein TP-NDG im Zeitfenster
		scf
		rra
		ld	(AZUST), a
;
		ld	b, 0		; Zhlerstand HP
		ld	a, c
		cpl
		ld	c, a		; Abstand zwischen letztem TP-NDG und Zeitfensterende
;
		ld	a, 10h		; Vektorpuffer tauschen
		xor	l
		ld	l, a
;

ZUL4:		ex	af, af'
		exx
INT0:		ei
		reti
;
ZUL1:		ccf
		rra
		ld	(AZUST), a
		ld	bc, 0		; Zhlerstand TP,HP zurcksetzen
; beide Zeitfenster verarbeiten
; 1. Zeitfenster
		ld	a, 10h		; Vektorpuffer tauschen
		xor	l
		ld	l, a
		call	PAUSE
		ld	a, (AZUST)
		and	3
		jr	nz, ZUL2	; XFELD	voll od. sicheres Wortende
; 2. Zeitfenster
		ld	a, 10h		; Vektorpuffer tauschen
		xor	l
		ld	l, a
		call	PAUSE
		ld	a, (AZUST)
ZUL5:		and	7
		jr	z, ZUL4
		and	3
		jr	z, ZUL3		; progn. Wortende
; Analyse abbrechen bei XFELD-berlauf oder sicherem Wortende
ZUL2:		out	(LEDOFF), a
		ex	af, af'
		exx
		reti
; Rcksprung zu (ADWEP) bei progn. Wortende
; Achtung: nach Rcksprung SP korrigieren
ZUL3:		ex	af, af'
		exx
		ex	(sp), hl
		ld	hl, ADWEP
		ex	(sp), hl
		ei
		reti
;
; Interrupt TP-NDG (Tiefpa-Nulldurchgang)
;
TP:		exx
		ex	af, af'
;
		ld	a, (AZUST)
;
		rla
		in	a, (CTC1)
		cpl			; A: Zhlwert ab ZF-Beginn
		jr	c, TP2		; ZF-Ende bereits erkannt
		sub	c		; A: NDG-Abstand = Increment
		ld	e, a		; E: Increment,	DE: Zeiger in K-Tab.
		add	a, c
		ld	c, a		; C: Zhlwert ab ZF-Beginn
		ld	a, (de)		; A: Kanal-Nr.
		cp	KGREN
		jr	nc, TP1		; HP-NDG

TP6:		add	a, l
		ld	l, a		; HL: Zeiger im	akt. Vektorpuffer
		ld	a, e		; A: Increment
		add	a, (hl)
		ld	(hl), a
;
		ld	a, 0F0h   	; Zeiger im Vektorpuffer rcksetzen
		and	l
		ld	l, a		; HL: XXX0
;
TP1:		ex	af, af'
		exx
		ei
		reti
; Zeitfensterende bereits erkannt
TP2:		ld	e, c
		ld	c, a		; C: neuer Zhlerstand TP
		add	a, e		; A: Gesamt-Increment
		jr	c, TP3		; NDG-Abstand>255, ungltig!
		ld	e, a		; E: Gesamt-Increment
		ld	a, (de)		; A: Kanal-Nr.
		cp	KGREN
		jr	c, TP4		; TP-NDG

TP3:		ld	a, 8		; NDG ungltig machen
TP4:		add	a, l
		ld	l, a		; HL: Zeiger in	Vektorpuffer
; Aktuelles Zeitfenster bearbeiten
		ld	a, c
		add	a, (hl)
		ld	(hl), a
; Altes Zeitfenster abschlieen
		ld	a, 10h		; Vektorpuffer tauschen
		xor	l
		ld	l, a
;
		ld	a, e		; A: Gesamt-Increment
		sub	c
		add	a, (hl)
		ld	(hl), a
;
		ld	a, 0F0h		; Zeiger im Vektorpuffer rcksetzen
		and	l
		ld	l, a
		call	PAUSE		; Vektor im XFELD abspeichern
		ld	a, 10h		; Vektorpuffer tauschen
		xor	l
		ld	l, a
;
		ld	a, (AZUST)
		res	7, a
		ld	(AZUST), a
		jr	ZUL5
;
;Interrupt HP-NDG (Hochpa-Nulldurchgang)
;
HP:		exx
		ex	af, af'
;
		in	a, (CTC1)
		cpl			; A: Zhlwert ab ZF-Beginn
		sub	b		; A: NDG-Abstand = Increment
		ld	e, a		; E: Increment,	DE: Zeiger in K.-Tab.
		add	a, b
		ld	b, a		; Zhlwert ab ZF-Beginn
		ld	a, (de)		; A: Kanal-Nr.
		cp	KGREN
		jr	c, TP1
		jr	TP6


;------------------------------------------------------------------------------
; Pausendetektion
;
; Abspeichern eines 8-Byte-Vektors aus dem Vektorpuffer
; als 1-Byte-Vektor auf der aktuellen Adresse im Signalspeicher
;
; 	HL-Reg.: Adresse des akt. Vektorpuffers
; 	XFELD:	Beginn Signalspeicher
; 	EXFEL:	Ende des Signalspeichers
; 	XA:	Aktuelle Adresse in XFELD
; 	VANZ:	Zahl der Vektoren bis Progn. Wortende
; 	SW1:	Minimale Ko.-Summe fr Signalanfangserkennung
; 	T1:	Mindestdauer fr Signalanfangserkennung
; 	SW2:	Maximale Ko.-Summe fr Pausenerkennung
; 	T2:	Maximale Dauer fr Pausenerkennung
; 	SW3:	Minimale Ko.-Summe fr Knackerkennung
; 	T3:	Maximale Knackdauer
; 	TT1:	Aktuelle Signaldauer nach Signalanfangserk., TT1=T1...0
; 	TT2:	Aktuelle Pausendauer nach Pausenerkennung., TT2=T2...-T3
; 	TT3:	Aktuelle Knackdauer nach Knackerk., TT3=T3...0
; 	AZUST:	Bit 4 gesetzt: progn. Wortanfang
; 		Bit 3 gesetzt: sicherer	Wortanfang
; 		Bit 2 gesetzt: progn. Wortende
; 		Bit 1 gesetzt: sicheres	Wortende
; 		Bit 0 gesetzt: XFELD-berlauf
;
;------------------------------------------------------------------------------

PAUSE:		push	hl
		push	de
		push	bc
; 1-Byte-Vektor und Komponentensumme ermitteln
		ld	a, SWHL
		ld	c, a		; C: HL-Schwellwert
		ld	e, 0
		ld	b, DIM 		; B: Komponentenzhler

PA10:		ld	a, (hl)
		ld	(hl), 0		; aktuellen Vektorpuffer lschen
		cp	c
		rr	d		; D: 1-Byte-Vektor, HL-Schwelle	berschr.=>0!
		add	a, e
		jr	nc, PA100	; Kein berlauf	der Komponentensumme
		ld	a, 0FFh

PA100:		ld	e, a
		inc	hl
		djnz	PA10
;
		ld	hl, (XA)
		ld	(hl), d
		inc	hl
		ld	(XA), hl
		ld	bc, EXFEL
		scf
		sbc	hl, bc
; Sprungverteiler
		ld	a, (AZUST)
		ld	c, a		; C: AZUST
		jp	z, PA000	; XFELD-berlauf
		and	1Eh		; Bit 1..4 ausblenden
		jr	z, PA1		; noch kein Signalanfang
		and	0Eh		; Bit 1..3 ausblenden
		jr	z, PA1		; Wortanf. progn., noch	kein sicherer Wortanf.
		and	6		; Bit 1..2 ausblenden
		jr	z, PA2		; sicherer Wortanfang bereits erkannt
; Wortende bereits prognostiziert
		ld	a, SW3
		sub	e
		ld	hl, TT2
		dec	(hl)
		jr	c, PA31		; KSUM.SW3
		ld	a, T3
		ld	(TT3), a
		jr	z, PA32		; TT2=T2
		jp	p, PA0		; TT2<T2
;
PA32:		set	1, c
		jr	PA0		; sicheres Wortende
; Knack&
PA31:		ld	hl, TT3
		dec	(hl)
		jr	nz, PA0		; TT3<T3
; TT3=T3, kein Knack
PA33:		res	2, c
		ld	a, T2
		ld	(TT2), a
		ld	a, T3
		ld	(TT3), a
		jr	PA0		; kein Wortende
;
; sicherer Wortanfang bereits erkannt
PA2:		ld	a, SW2
		sub	e
		jr	c, PA0		; KSUM.SW2
		set	2, c		; progn. Wortende
		ld	hl, TT2
		dec	(hl)
		jr	nz, PA00	; TT2<T2, progn. Wortende
		set	1, c		; sicheres Wortende
		jr	PA00		; sicheres Wortende bei	T2=1
;
; noch kein Signal erkannt oder Wortanfang nur prognostiziert
PA1:		ld	a, SW1
		sub	e
		jr	c, PA11		; KSUM>SW1
		ld	hl, XFELD
		ld	(XA), hl
		res	4, c
		ld	a, T1
		ld	(TT1), a
		jr	PA0		; noch kein Wortanfang
;
PA11:		set	4, c		; progn. Wortanfang
		ld	hl, TT1
		dec	(hl)
		jr	nz, PA0		; TT1<T1
		set	3, c		; sicherer Wortanfang
		jr	PA0		; progn. Wortanfang, sicherer Wortanfang bei T1=1
;
PA000:		set	0, c		; XFELD-berlauf
; Berechnung der Vektoranzahl im XFELD
; Bei XFELD-berlauf oder progn. Wortende
; Bei XFELD-berlauf: VANZ=EXFELD-XFELD!
PA00:		scf
		ld	hl, (XA)
		ld	de, XFELD
		sbc	hl, de
		ld	a, l
		ld	(VANZ),	a
;

PA0:		ld	a, c
		ld	(AZUST), a
		pop	bc
		pop	de
		pop	hl
		ret

;------------------------------------------------------------------------------
; Klassen loeschen
;------------------------------------------------------------------------------

mDelClass:	call	OUTCRLF		; 1x Enter ausgeben
mDelClass1:	ld	de, aKlassennummer ; "Klassennummer:    \b\b\b\b"
		call	STROUT		; Textausgabe
		ld	a, 3
		call	ETAST		; Texteingabe nach DE
		ret	c
		jr	z, mDelClass4
		call	DEZHX		; Umwandlung BCD-Zahl (A Stellen Text ab DE) nach HL
		jr	c, mDelClass4
		and	a
		jr	z, mDelClass4
		call	sub_A1B5
		jr	nz, mDelClass5
		ld	(XSYM),	a
mDelClass2:	ld	de, aKlasseLoeschen ; "Klasse loeschen J/N ?  \b\b"
		call	STROUT		; Textausgabe
		ld	a, 2
		call	ETAST		; Texteingabe nach DE
		ret	c
		cp	1
		jr	nz, mDelClass6
		ld	a, (de)
		cp	'J'
		jr	z, mDelClass3
		cp	'Y'
		jr	z, mDelClass3
		cp	'N'
		jr	z, mDelClass
		jr	mDelClass6
;
mDelClass3:	ld	a, 0FFh
		ld	(NSCHW), a
		call	CLST
		ld	a, SWCLU
		ld	(NSCHW), a
		jr	mDelClass
;
mDelClass4:	call	CUUP		; Cursor hoch
		jr	mDelClass1
;
mDelClass5:	ld	de, aKlasseNichtVor ; "Klasse nicht vorhanden\r\n"
		call	STROUT		; Textausgabe
		call	BEEP
		jr	mDelClass1
;
mDelClass6:	call	CUUP		; Cursor hoch
		jr	mDelClass2

aKlassennummer:	db	"Klassennummer:    \b\b\b\b",0
aKlasseNichtVor:db	"Klasse nicht vorhanden\r\n",0
aKlasseLoeschen:db	"Klasse loeschen J/N ?  \b\b",0



sub_A1B5:	ld	b, a
		ld	a, (SPRIN)	; Sprecherindex
		add	a, b

sub_A1BA:	ld	hl, ANSYM+1
		ld	bc, (KLAZ)	; Anzahl der Cluster
		ld	b, 0
		inc	c
		cpir
		ret


;------------------------------------------------------------------------------
; Klassen umbenennen
;------------------------------------------------------------------------------

mRename:	call	OUTCRLF		; 1x Enter ausgeben
mRen1:		ld	de, aKlassennummerA ; "Klassennummer alt:    \b\b\b\b"
		call	STROUT		; Textausgabe
		ld	a, 3
		call	ETAST		; Texteingabe nach DE
		ret	c
		jr	z, mRen2
		call	DEZHX		; Umwandlung BCD-Zahl (A Stellen Text ab DE) nach HL
		jr	c, mRen2
		and	a
		jr	z, mRen2
		call	sub_A1B5
		jr	z, mRen4
		jr	mRen3
;
mRen2:		call	CUUP		; Cursor hoch
		jr	mRen1
;
mRen3:		ld	de, aKlasseNichtVor ; "Klasse nicht vorhanden\r\n"
		call	STROUT		; Textausgabe
		call	BEEP
		jr	mRen1
;
mRen4:		push	af
		pop	iy
mRen5:		ld	de, aNeu	; "              neu:    \b\b\b\b"
		call	STROUT		; Textausgabe
		ld	a, 3
		call	ETAST		; Texteingabe nach DE
		ret	c
		jr	z, mRen7
		call	DEZHX		; Umwandlung BCD-Zahl (A Stellen Text ab DE) nach HL
		jr	c, mRen7
		and	a
		jr	z, mRen7
		call	sub_A1B5
		jr	z, mRen8
		ld	d, a
		push	iy
		pop	af
		call	sub_A1BA
mRen6:		jp	po, mRename
		dec	hl
		ld	(hl), d
		inc	hl
		cpir
		jr	mRen6
;
mRen7:		call	CUUP		; Cursor hoch
		jr	mRen5
;
mRen8:		ld	de, aKlasseSchonVor ; "Klasse schon vorhanden\r\n"
		call	STROUT		; Textausgabe
		call	BEEP
		jr	mRen5

aKlassennummerA:db	"Klassennummer alt:    \b\b\b\b",0
aNeu:		db	"              neu:    \b\b\b\b",0
aKlasseSchonVor:db	"Klasse schon vorhanden\r\n",0

;------------------------------------------------------------------------------
; Wortschatz abspeichern
;------------------------------------------------------------------------------

mSave:		call	filename	; Dateiname erfragen und in FCB	eintragen
		ret	c
		ld	hl, KLAZ	; Anzahl der Cluster
		ld	(6Dh), hl
		ld	hl, (KLAZ)	; Anzahl der Cluster
		ld	h, 0
		add	hl, hl
		add	hl, hl
		add	hl, hl
		add	hl, hl
		ld	de, unk_A821
		add	hl, de
		ld	(6Fh), hl
		ld	hl, 0FFFFh
		ld	(71h), hl
		ld	c, 15		; OPENW	Erffnen Kassette schreiben
		call	5
		jr	nc, mSave1
		and	a
		ret	z
		jp	OUT_ENTER	; Ausgabe von ">ENTER<" und Eingabe ein Zeichen
;
mSave1:		ld	e, ' '
		call	TAUBD		; Zeichenausgabe E
		ld	de, KLAZ	; Anzahl der Cluster
		ld	c, 26		; SETDM	Setzen der Pufferadresse fr Kassette lesen/schreiben
		call	5
		ld	hl, (6Fh)
		inc	hl
		and	a
		sbc	hl, de
		sla	l
		rl	h
		ld	a, l
		and	a
		jr	nz, mSave2
		dec	h
mSave2:		ld	b, h
		ld	(RETT),	bc
mSave3:		ld	c, 21		; WRITS	Schreiben eines	Blockes	auf Kassette
		call	5
		jp	c, OUT_ENTER	; Ausgabe von ">ENTER<" und Eingabe ein Zeichen
		ld	e, ' '
		call	TAUBD		; Zeichenausgabe E
		djnz	mSave3
		ld	c, 16		; CLOSW	Abschlieen Kassette schreiben
		call	5
		jp	c, OUT_ENTER	; Ausgabe von ">ENTER<" und Eingabe ein Zeichen
		ld	de, aVerifyYN	; "\r\nVerify (Y)/N ?"
		call	STROUT		; Textausgabe
		ld	a, 1
		call	ETAST		; Texteingabe nach DE
		ret	c
		jr	z, mSave4
		ld	a, (de)
		cp	'Y'
		jr	z, mSave4
		cp	'J'
		ret	nz
;
mSave4:		ld	de, aRewind	; "\r\nRewind !\r\n"
		call	STROUT		; Textausgabe
		ld	c, 1
		call	5
		cp	3
		ret	z
;
mSave5:		ld	de, CONBU
		ld	c, 26		; SETDM	Setzen der Pufferadresse fr Kassette lesen/schreiben
		call	5
		ld	c, 13		; OPENR	Erffnen Kassette lesen
		call	5
		jr	nc, mSave6
		and	a
		ret	z
		call	0F5A6h		; OS: REA1, WARTEN AUF BEDIENERREAKTION
		ret	c
		jr	mSave5
;
mSave6:		ld	bc, (RETT)
		inc	b
mSave7:		ld	de, CONBU
		ld	c, 26		; SETDM	Setzen der Pufferadresse fr Kassette lesen/schreiben
		call	5
		ld	c, 20		; READS	Lesen eines Blockes von	Kassette
		call	5
		jr	c, mSave9
		dec	b
		jr	z, mSave8
		and	a
		jr	z, mSave7
		jp	mLoad8
;
mSave8:		and	a
		jp	z, mLoad8
		ret
;
mSave9:		call	0F5A6h		; OS: REA1, WARTEN AUF BEDIENERREAKTION
		ret	c
		jr	mSave7
aVerifyYN:	db	"\r\n"
		db	"Verify (Y)/N ?",0
aRewind:	db	"\r\n"
		db	"Rewind !\r\n",0

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

; Cursor hoch
CUUP:		ld	e, 0Bh		; Cursor hoch ohne Zeichenlschen
		ld	c, 2		; CONSO	Ausgabe	eines Zeichens zu CONST
		call	5


; Erzeugung eines Tones
BEEP:		push	bc
		push	de
		push	af
		ld	a, 27h ; '''
		out	(0E2h),	a
		ld	a, 4
		out	(0E2h),	a
		ld	e, 7		; Ausgabe eines	Summertones
		ld	c, 2		; CONSO	Ausgabe	eines Zeichens zu CONST
		call	5
		ld	a, 3
		out	(0E2h),	a
		pop	af
		pop	de
		pop	bc
		ret

; 1x Enter ausgeben
OUTCRLF:	ld	de, aCRLF	; "\r\n"
		jr	STROUT		; Textausgabe
; 2x Enter ausgeben
OUT2CRLF:	ld	de, a2CRLF	; "\r\n"
		jr	STROUT		; Textausgabe

a2CRLF:		db	"\r\n"
aCRLF:		db	"\r\n",0

OUT_ENTER:	call	BEEP		; Ausgabe von ">ENTER<" und Eingabe ein Zeichen
		ld	de, aEnter	; "\r\n>ENTER<"
		call	STROUT		; Textausgabe
		ld	a, 1
		jp	ETAST		; Texteingabe nach DE

aEnter:		db	"\r\n"
		db	">ENTER<",0

; Textausgabe
STROUT:		ld	c, 9		; PRNST	Ausgabe	einer Zeichenkette zu CONST
jp_BDOS:	jp	5

; Bildschirm lschen
CLS:		ld	e, 0Ch		; Bildschirm lschen
		jr	TAUBD		; Zeichenausgabe E


;------------------------------------------------------------------------------
; Anzeige Text Reg. DE und Anzeige Reg.	A dezimal
;------------------------------------------------------------------------------

ANZM:		call	STROUT		; Textausgabe
		call	HEXDE		; Wandlung einer Hex-Zahl (Reg A) nach BCD (Reg	HL)
		ld	a, h
		and	0Fh
		jr	z, anzm1
		call	ZZEI1
		jr	anzm2
;
anzm1:		ld	a, l
		and	0F0h
		jr	z, anzm3
;
anzm2:		ld	a, l
		rrca
		rrca
		rrca
		rrca
		call	ZZEI1
;
anzm3:		ld	a, l
		call	ZZEI1
		ld	e, 20h ; ' '
		jr	TAUBD		; Zeichenausgabe E
;
ZZEI1:		and	0Fh
		add	a, 30h ; '0'
		ld	e, a

;------------------------------------------------------------------------------
; Zeichenausgabe E
;------------------------------------------------------------------------------

TAUBD:		ld	c, 2		; CONSO	Ausgabe	eines Zeichens zu CONST
		jr	jp_BDOS


;------------------------------------------------------------------------------
; Wandlung einer Hex-Zahl (Reg A) nach BCD (Reg	HL)
;------------------------------------------------------------------------------

HEXDE:		ld	ix, hexd_tab
		ld	hl, 0
		ld	c, 3
;
hexde1:		sub	(ix+0)
		jr	c, hexde2
		inc	l
		jr	hexde1
;
hexde2:		dec	c
		ret	z
		add	a, (ix+0)
		inc	ix
		add	hl, hl
		add	hl, hl
		add	hl, hl
		add	hl, hl
		jr	hexde1

hexd_tab:	db 100
		db 10
		db 1

;------------------------------------------------------------------------------
; Eingabe von der Tastatur
; Return: A: Lnge Zeichenkette
; 	  DE: Adresse Zeichenkette
;------------------------------------------------------------------------------

ETAST:		ld	de, CONBU   	; Texteingabe nach DE
		ld	(de), a
		ld	c, 10		; RCONB	Eingabe	einer Zeichenkette von CONST
		call	5
		ret	c
		inc	de
		ld	a, (de)
		inc	de
		and	a
		ret

;------------------------------------------------------------------------------
; Umwandlung einer BCD-Zahl (A Stellen Text ab DE) nach HL
; In:  A:  Anzahl Stellen
;      DE: Zeigt auf Text
; Out: HL: Hex-Zahl
;      Cy: bei Fehler (Zahl>255 (?)
;------------------------------------------------------------------------------

DEZHX:		ld	b, a
		ld	hl, 0
;
dezhx1:		push	bc
		add	hl, hl
		push	hl
		add	hl, hl
		add	hl, hl
		pop	bc
		add	hl, bc
		ld	a, (de)
		inc	de
		sub	30h ; '0'
		cp	0Ah
		pop	bc
		ccf
		ret	c
		push	bc
		ld	c, a
		ld	b, 0
		add	hl, bc
		pop	bc
		djnz	dezhx1
		ld	a, h
		and	a
		scf
		ret	nz
		ld	a, l
		cp	33h ; '3'
		ccf
		ret



		db	"F"
		db	"4123456789ABCDEF"
		db	"5123456789ABCDEF"
		db	"6123456789ABCDEF"
		db	"7123456789ABCDEF"

		org	ANF+0C80H

; vorberechnete Tabelle	der Hamming-Distanzen (1/4K-Grenze)
HTAB:		db	0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4 
		db	1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5
		db	1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5
		db	2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6
		db	1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5
		db	2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6
		db	2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6
		db	3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7
; NDG-Histogramm (an 256 Byte-Grenze!)
KTAB:		db	8, 7, 7, 6, 6, 5, 5, 4, 4, 4, 3, 3, 3, 3, 2, 2
		db	2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		db	1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 8
		db	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8
		db	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8
		db	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8
		db	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8
		db	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8
		db	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8
		db	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8
VPUF1:		db	"1234567890"
		db	"123456"
		db	"123456789"
loadflg:	db	1
NSCHW:		db	SWCLU
INDEX:		db	0
twz:		db	1		; eingestellter Teilwortschatz
SPRIN:		db	0		; Sprecherindex
XA:		dw	0
AZUST:		db	0
NSUM:		db	0
SPRI1:		db	0
IDEX1:		db	0
ZZ:		db	"12345678"
RETT:		dw	0
XDIFF:		db	0
YDIFF:		db	0
VANZ:		db	0
MINHD:		db	0
TT1:		db	0
TT2:		db	0
TT3:		db	0
YFELD:		db	"1234567890"
		db	"123456"
XFELD:		db	"1234567890"
		db	"1234567890"
		db	"1234567890"
		db	"1234567890"
		db	"1234567890"
		db	"1234567890"
		db	"1234567890"
EXFEL:		db	1
XSYM:		db	0
KLAZ:		db	0		; Anzahl der Cluster
ANSYM:		db	0, 0FFh
		ds	199
unk_A757:	ds	1
		ds	1
BFELD:		ds	200
unk_A821:	ds	1
RFELD:		ds	0CEBH
SPERK:		ds	1

		end
